# QEMU - For Redox OS

modifications author: tarun.aditya@hotmail.com (mattermost chat username: enygmator)

This folder contain the file `recipe.sh` which is used to build the QEMU repo (version 7.0.0), modified to run on Redox OS. The steps for compilation and execution on Redox OS are given in the QEMU repo's README. In this file, we elaborate on the process to help you resolve any bugs you encounter.

You can download the QEMU repo from [this link: enygmator/qemu](https://gitlab.redox-os.org/enygmator/qemu), or if you want the latest version (which is recommended at the moment), from [here](https://gitlab.redox-os.org/enygmator/qemu/-/tree/qemu_for_redox).

The "recipe" to "cook" this program/`package` can be found in the Redox OS "cookbook", under the folder `qemu` (`$TopLevel/redox/cookbook/recipes/qemu/`), which is the folder in which this `README.md` is originally present.

## Some info about the current state of QEMU compilation

1. only `qemu-system-x86_64` is built
2. A static binary is built, by manually linking libraries; to work with the `-static` flag of linker
3. We will enable only the features of QEMU that we need. Any default features that are disabled will be mentioned in [this wiki](https://gitlab.redox-os.org/enygmator/qemu/-/wikis/Features-of-QEMU-on-Redox/)


## Compilation and Execution

1. Assume you have Redox OS in the path `$TopLevel/redox`, where `$TopLevel` can be any path.
2. Place the QEMU repo in a folder named `qemu-7.0.0`. If you change the location of the repo, or the folder name, edit the variables `VERSION` and `QEMU_PATH` in `recipe.sh`.
3. Go to `$TopLevel/redox` in the terminal and run the command `make cook PKG=qemu`.
   NOTE: Before running the command, it would be better if you have compiled and run Redox OS at least once (since, `prefix` is required to have been created correctly for this command to work).
   > The `make cook` command will work regardless of whether this package will be included in the final buils of `harddrive.bin`.
4. If the command runs successfully, then you can proceed to include the "cooked" package, in the final `harddrive.bin` that emulated the harddrive of Redox OS, when run in QEMU.
   To do that, you'll simply have to include the line `qemu = {}` in `filesystem.toml`, under the "table" `[packages]`.
   > `filesystem.toml` points to one of the files in the `config/` folder.
5. Now, it's time to compile and generate `harddrive.bin` with Redox OS and QEMU installed on it, and load it into QEMU.
   To do that, go to `$TopLevel/redox` in the terminal and execute:
   ```
   touch filesystem.toml
   make qemu
   ```
   > You need to run `touch filesystem.toml` if you want the `harddrive.bin` to be regenerated (which includes building packages/recipes which have changed)
6. Inside `Redox OS`, you can open the terminal and run the command `qemu-system-x86_64`, which will open up QEMU. You can use "regular" QEMU commands, including an emulation involving GUI using the SDL2 GUI Library. At the moment, it fails to launch.

## The compilation procedure

The following steps take place during the cooking of this recipe:
1. **fetch**: `recipe_fetch` is run and `source` folder is created (it will remain empty)
2. **prepare**: `recipe_prepare` is run where:
   - all `BUILD_DEPENDS` which are required for static compilation of dependencies are compiled and their resulting artifacts (Libraries and headers) put into `sysroot` folder
   - The `build` folder is created and setup, which in this case is left empty (for now).
3. **build -> stage -> tar**:
   - The **build** step does 2 things:
      - `recipe_build` function is run, where the `$QEMU_PATH/configure` file from the `QEMU_PATH` source is run, which uses `VPATH` (meaning that, when building, the source files aren't modified, rather, required modification are carried out in copies of those files which are put in the present `build` folder. The modified files in this `build` folder refer to unchanged files back in the course folder as required)
      - Once the `configure` is run, it creates `build` files (like `make` and `ninja` and `meson`) with respect to the features/configurations we requested. Now, it's time to actually **build** our custom version of QEMU by running the build system, which is the command `make`.
   - The **stage** step involves "installation" of the binaries created in the **build** step byrunning `make install`, with the destination set as the `stage` folder.
   - The **TAR** step, archives (zips) the `stage` folder to create `stage.tar.gz` and a `stage.toml`, which has information about the package created.
4. **pkg**: The `pkgar` tool (in cookbook) is used to create a `stage.pkgar`, which is a package archive file specific to Redox OS, along with a `stage.sig` (I'm not sure as to exactly what it does).
5. **publish**: is run after all recipes have completed until the previous step. All `stage.*` files are copied into `cookbook/repo` folder, which is **later** used to install stuff into `harddrive.bin`.

> All of the above steps are handled performed by `repo.sh`, which commands `cook.sh` to do whichever step needs to be performed, in the right order.

## Modifications to the package

When you make modifications such as the following...:
- to the repo (`qemu-7.0.0`)
- remove/include a new feature or change some setting by changing the options provided to `$QEMU_PATH/configure`

...you will need to rebuild things. Now, many a times, you'll not need to rebuild everything in the `build` folder as `configure` is capable of rebuilding only stuff that's changes, saving many minutes of complete compilation.

So, a **shortcut you can use to trigger a partial rebuild, which is sufficient in most cases, is by deleting the `stage.tar.gz` file**. A complete rebuild can be triggered by deleting the `build` folder.

Why does this work? As per the method of compilation at the time of writing, when the output file/folder of one stage is deleted, that particular stage and all subsequent stages are re-executed. The following are the output files/folders of various stages, which you may delete, to trigger re-execution (which will lead to them being rebuilt).

1. **fetch**: `source/`
2. **prepare**:
   - `build/`
   - `sysroot/`
   > if either doesn't exist, both folder are deleted and rebuilt
3. **build -> stage -> tar**: `stage.tar.gz`
4. **pkg**: `stage.pkgar`
5. **publish**: If `cookbook/repo/stage.tar.gz` is older than the output of **pkg** stage, this stage will be re-executed.


> I, Enygmator, have plan to have a "commander" app that will help you perform "basic tasks" in one go, via pre-programmed steps of execution. But this may not be well into the future.

