#7.0.0 (19-Apr-2021) is the latest version for the year in which dev on qemu_on_redox started (12-May-2021)
VERSION=7.0.0
TAR=https://download.qemu.org/qemu-$VERSION.tar.xz

#temporary: source QEMU source path
QEMU_PATH=$ROOT/../../redox_apps/qemu-$VERSION/

#Dependencies of this build on other packages available in Redox OS
BUILD_DEPENDS=(curl glib libiconv libjpeg libpng pcre pixman sdl sdl2 zlib liborbital nghttp2 openssl gettext mesa)
# ncurses ncursesw

#this function is run if source folder doesn't exist
function recipe_fetch {
    #TODO: Temporarily use direct path to QEMU directory, instead of downloading TAR

    #temporarily disabling copying, since `configure` uses VPATH (separate build tree)
    mkdir source
    #cp -rp "$ROOT/../../qemu-7.0.0" "source"

    #Since we are copying, we skip the download of TAR
    skip=1
}

#TODO: This is a temporary function to make sure that time is saved by copying from local folder to build
#this function is run if `build` folder doesn't exist
function recipe_prepare {
    rm -rf sysroot
    mkdir sysroot

    if [ ${#BUILD_DEPENDS} -gt 0 ]
    then
        pushd $ROOT
            ./repo.sh "${BUILD_DEPENDS[@]}"
        popd

        for i in "${BUILD_DEPENDS[@]}"
        do
            pkgar \
                extract \
                sysroot \
                --archive "$REPO/$i.pkgar" \
                --pkey "${ROOT}/build/id_ed25519.pub.toml"
        done
    fi

    rm -rf build
    mkdir build # we just need an empty directory (Skip build process as we use configure+VPATH)

    #skip default `prepare` function
    skip=1
}

function recipe_version {
    echo "$VERSION"
    skip=1
}

function recipe_update {
    echo "skipping update"
    skip=1
}

function recipe_build {

    sysroot="$(realpath ../sysroot)"

    # `--divide` is used disable interpreting `/` as "comments" in the assembler (AS)
    export CFLAGS="-I$sysroot/include -Xassembler --divide"
    export CPPFLAGS="-I$sysroot/include -Xassembler --divide"
    export CXXFLAGS="-I$sysroot/include -Xassembler --divide"
    export LDFLAGS="-L$sysroot/lib -lOSMesa -Xassembler --divide"
    export ASFLAGS="--divide"


    # We will enable only the features of QEMU that we need.
    # Any default features that are disabled will be mentioned below:
    # https://gitlab.redox-os.org/enygmator/qemu/-/wikis/Features-of-QEMU-on-Redox/

    # $BUILD: machine on which qemu is built (this machine)
    # $HOST: machine on which qemu will run (redox)

    # `prefix` and related paths refer to the location of stuff in redoxfs

    $QEMU_PATH/configure \
        --prefix="/" \
        --bindir="bin" \
        --datadir="share" \
        --includedir="include" \
        --libdir="lib" \
        --libexecdir="libexec" \
        --mandir="share/man" \
        --docdir="share/doc" \
        --localedir="libexec" \
        --sysconfdir="etc" \
        \
        --target-list=x86_64-softmmu \
        --without-default-features \
        --cross-compile-to-redox="yes" \
        --disable-slirp \
        --disable-capstone \
        --disable-coroutine-pool \
        --enable-curl \
        --enable-tcg \
        --disable-pie \
        --enable-debug \
        --static \
        --enable-sdl \
        #--audio-drv-list=sdl \
        #--enable-curses \

        #--without-default-devices \

    echo -e "\e[1;33;44m CONFIGURED QEMU \e[0m"

    "$REDOX_MAKE" -j"$($NPROC)"

    echo -e "\e[1;33;44m MADE QEMU \e[0m"

    skip=1
}

function recipe_test {
    echo "skipping test"
    skip=1
}

function recipe_clean {
    "$REDOX_MAKE" clean
    skip=1
}

function recipe_stage {

    dest="$(realpath $1)"
    "$REDOX_MAKE" DESTDIR="$dest" install

    echo -e "\e[1;33;44m INSTALLED QEMU \e[0m"

    skip=1
}
